--- computations for simplex construction T

needsPackage "Polyhedra"

-- matrix of vertices determined using the construction given in the paper
T3 = matrix{{-3,1,1,1},{-2,0,2,0},{-2,0,0,2}}; -- 3d
T4 = matrix{{-3,1,1,1,1},{-2,0,2,0,0},{-2,0,0,2,0},{-2,0,0,0,4}}; -- 4d

-- determine matrix A with all-1-vector and nonnegative entries
matrixA = M -> (
    P = convexHull M;
    L = latticePoints P;
    L = matrix{L};
    n = numgens source L;
    d = numgens target L;
    seqn = 0..n-1;
    seqd = 0..d-1;
    listn = toList seqn;
    listd = toList seqd;
    listd = apply(listd, i -> min apply(listn, j -> L_(i,j)));
    T = matrix table(d,n,(i,j) -> -listd_i);
    L = L + T;
    E = transpose matrix vector(apply(listn, i -> 1));
    L = E||L
)

-- determine ml equations
MLeqs = {coeffRing=>QQ} >> o -> (expr) -> (
    m = numgens source expr;
    R := (o.coeffRing)[vars {53..(52+(numgens target expr))},u_1..u_m];
    A := substitute(expr,R);
    U = u_1..u_m;
    U = toList U;
    U = transpose matrix {for i in U list i_R};
    N = sum for i from 1 to m list u_i;
    X := for i in 0..(m-1) list R_(entries expr_i);
    X = transpose matrix {X};
    system := ideal (A*((N * X) - U));
    return first entries gens system
)

-- write list of polynomials in julia file  
SystemT3 = MLeqs matrixA T3
SystemT4 = MLeqs matrixA T4

polynomialjl = p -> (
    replace("_1","[1]",replace("_2","[2]",replace("_3","[3]",replace("_4","[4]",replace("_5","[5]",replace("_6","[6]",
    replace("_7","[7]",replace("_8","[8]",replace("_9","[9]",replace("_10","[10]",replace("_11","[11]",
    replace("_12","[12]",replace("_13","[13]",replace("_14","[14]",replace("_15","[15]",replace("_16","[16]",
    replace("_17","[17]",replace("_18","[18]",replace("_19","[19]",replace("_20","[20]",replace("_21","[21]",toExternalString p)))))))))))))))))))))
)

writeSystem = method(Options=>{})

writeSystem (List, File) := o -> (P,f) -> (
    variables = {"x1","x2","x3","x4","x5","u[1:21]"};
    f << "## This file was generated by Macaulay2 to compute the ML degree via homotopy continuation.\n\n";
    f << "using HomotopyContinuation";
    f << "\n\n";
    f << "@var " | concatenate(mingle(variables, #variables:1));
    eqnCommas:=(#P-1):", ";
    f << "\n\n";
    f << "F = System(\n";
    f << "    [\n";
    f << "         "|concatenate mingle(apply(P,e->polynomialjl e), eqnCommas);
    f << "\n";
    f << "    ];\n";
    f << "    parameters = u\n"  ;
    f << ")\n\n";
    f << "## Solve generically:\n\n";
    f << "u0 = rand(ComplexF64,21)\n";
    f << "result_u0 = solve(F, target_parameters = u0)\n\n";
    f << "## Solve for random parameter values\n\n";
    f << "data = [rand((0:100000),21)]\n";
    f << "data_points = solve(\n";
    f << "      F,\n";
    f << "      solutions(result_u0),\n";
    f << "      start_parameters =  u0,\n";
    f << "      target_parameters = data,\n";
    f << ")";
)

writeSystem (List, String) :=o -> (P,filename) -> (
    f := openOut filename;
    writeSystem(P,f);
    close f;
)

writeSystem(SystemT3, "simplexT3d.jl")
writeSystem(SystemT4, "simplexT4d.jl")